'use strict';

/**
 *
 *
 * @author Mai
 * @date 2018/8/14
 * @version
 */

const audit = require('../const/audit');

module.exports = app => {
    class ChangeAudit extends app.BaseService {
        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'change_audit';
        }

        /**
         * 获取最后一位审批人
         * @param {int} cid - 变更令id
         * @param {int} times - 次数
         * @param {int} site - 位置
         * @param {int} status - 状态
         * @return {void}
         */
        async getLastUser(cid, times, site = 1, status = 1) {
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('cid', {
                value: this.db.escape(cid),
                operate: '=',
            });
            this.sqlBuilder.setAndWhere('times', {
                value: times,
                operate: '=',
            });
            if (status === 1) {
                this.sqlBuilder.setAndWhere('status', {
                    value: 1,
                    operate: '!=',
                });
            }
            if (site === 0) {
                this.sqlBuilder.setAndWhere('usite', {
                    value: site,
                    operate: '=',
                });
            }
            const u_sort = [['usort', 'DESC']];
            this.sqlBuilder.orderBy = u_sort;
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
            const data = await this.db.queryOne(sql, sqlParam);

            return data;
        }

        /**
         * 获取最后一位退回的审批人
         * @param {int} cid - 变更令id
         * @param {int} times - 次数
         * @return {void}
         */
        async getLastBackUser(cid, times) {
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('cid', {
                value: this.db.escape(cid),
                operate: '=',
            });
            this.sqlBuilder.setAndWhere('times', {
                value: times,
                operate: '=',
            });
            this.sqlBuilder.setAndWhere('status', {
                value: 6,
                operate: '=',
            });
            const u_sort = [['usort', 'DESC']];
            this.sqlBuilder.orderBy = u_sort;
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
            const data = await this.db.queryOne(sql, sqlParam);

            return data;
        }

        /**
         * 获取当前审批人查看info时的状态
         * @param {Object} change - 变更令数据
         * @return {void}
         */
        async getStatusByChange(change) {
            const statusConst = audit.flow.status;
            const auditStatusConst = audit.flow.auditStatus;
            const uid = this.ctx.session.sessionUser.accountId;
            const changeAuditInfo = await this.getAllDataByCondition({ where: { cid: change.cid, times: change.times, uid }, orders: [['id', 'desc']], limit: 1, offset: 0 });
            if (changeAuditInfo === null || changeAuditInfo[0].status === undefined) {
                // 无权限查看此变更令
                return 0;
            }

            if (change.status === statusConst.uncheck && uid === change.uid) {
                // 待上报
                return 1;
            } else if (change.status === statusConst.back && uid === change.uid) {
                // 待重新上报
                return 2;
            } else if (change.status === statusConst.back && uid !== change.uid) {
                // 被退回但你不是原报人
                return 3;
            } else if (change.status === statusConst.checked) {
                // 已完成
                return 4;
            } else if (change.status === statusConst.checkNo) {
                // 已终止
                return 5;
            } else if ((change.status === statusConst.checking || change.status === statusConst.backnew) && changeAuditInfo[0].status === auditStatusConst.checking) {
                // 待你审批
                return 6;
            } else if ((change.status === statusConst.checking || change.status === statusConst.backnew) && changeAuditInfo[0].status !== auditStatusConst.checking) {
                // 审批中但你未到你审批或你已审批
                return 7;
            }
            // 无权限查看此变更令
            return 0;
        }

        /**
         * 根据用户查看此变更状态获取审批人列表
         * @param {Object} change - 变更令
         * @param {int} status - 状态
         * @return {object} list - 列表
         */
        async getListByStatus(change, status) {
            let sql = '';
            let sqlParam = '';
            switch (status) {
                case 1:// 待上报
                case 2:// 待重新上报
                    sql = 'SELECT * FROM ?? WHERE ' +
                        'cid = ? AND times = ? GROUP BY usite';
                    sqlParam = [this.tableName, change.cid,
                        change.times];
                    break;
                case 3: // 被退回但你不是原报人
                case 4:// 已完成
                case 5:// 已终止
                case 7:// 审批中但你未到你审批或你已审批
                    // 获取完整的审批顺序
                    sql = 'SELECT * FROM ?? WHERE ' +
                        'cid = ? ORDER BY usort';
                    sqlParam = [this.tableName, change.cid];
                    break;
                case 6: // 审批中
                    sql = 'SELECT * FROM (SELECT MAX(usort) as ust FROM ?? ' +
                        'WHERE cid = ? and times = ? GROUP BY usite ) as b ' +
                        'JOIN ?? as a ON a.usort = b.ust WHERE cid = ? and times = ? ORDER BY usite';
                    sqlParam = [this.tableName, change.cid, change.times, this.tableName, change.cid, change.times];
                    break;
                default:// 无权限查看此变更令
                    break;
            }
            const list = await this.db.query(sql, sqlParam);
            return list;
        }

        /**
         * 删除变更令审批人列表
         * @param {Object} transaction - 事务
         * @param {Object} cid - 变更令id
         * @param {int} times - 次数
         * @return {object} 返回结果
         */
        async deleteAuditData(transaction, cid, times) {
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('cid', {
                value: this.db.escape(cid),
                operate: '=',
            });
            this.sqlBuilder.setAndWhere('times', {
                value: times,
                operate: '=',
            });
            this.sqlBuilder.setAndWhere('usite', {
                value: 0,
                operate: '!=',
            });
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName, 'delete');
            const result = await transaction.query(sql, sqlParam);

            return result;
        }

        /**
         * 获取不重复列表
         * @param {Object} cid - 变更令id
         * @param {int} times - 次数
         * @return {object} 返回结果
         */
        async getListGroupByTimes(cid, times) {
            const sql = 'SELECT * FROM ?? WHERE ' +
                'cid = ? AND times = ? GROUP BY usite';
            const sqlParam = [this.tableName, cid, times];
            const list = await this.db.query(sql, sqlParam);
            return list;
        }

        /**
         * 获取比sort大的审批人列表
         * @param {Object} cid - 变更令id
         * @param {int} usort - 排序
         * @return {object} 返回结果
         */
        async getNextAuditList(cid, usort) {
            const sql = 'SELECT * FROM ?? WHERE ' +
                'cid = ? AND usort > ?';
            const sqlParam = [this.tableName, cid, usort];
            const list = await this.db.query(sql, sqlParam);
            return list;
        }

        /**
         * 获取除当前次数的审批人列表
         * @param {Object} cid - 变更令id
         * @param {int} times - 次数
         * @return {object} 返回结果
         */
        async getListByBack(cid, times) {
            this.initSqlBuilder();
            this.sqlBuilder.setAndWhere('cid', {
                value: this.db.escape(cid),
                operate: '=',
            });
            this.sqlBuilder.setAndWhere('times', {
                value: times,
                operate: '!=',
            });
            const [sql, sqlParam] = this.sqlBuilder.build(this.tableName);
            const result = await this.db.query(sql, sqlParam);

            return result;
        }

        /**
         * 获取 审核人 待审批的（） 变更令列表
         * @param uid
         * @returns {Promise<void>}
         */
        async getAuditChange(uid) {
            const sql = 'SELECT ca.`uid`, ca.`times`, ca.`usite`, ca.`usort`, ca.`tid`, ca.`cid`, ca.`sin_time`, ' +
                '    c.`code` As `ccode`, c.`name` As `cname`,' +
                '    t.`name`, t.`type`, t.`user_id` ' +
                '  FROM ?? AS ca, ?? AS c, ?? As t ' +
                '  WHERE ca.`uid` = ? and ca.`status` = ?' +
                '    and ca.`cid` = c.`cid` and ca.`tid` = t.`id`';
            const sqlParam = [this.tableName, this.ctx.service.change.tableName, this.ctx.service.tender.tableName, uid, 2];
            return await this.db.query(sql, sqlParam);
        }
    }

    return ChangeAudit;
};
