'use strict';

/**
 *
 *
 * @author Mai
 * @date
 * @version
 */

const measureType = require('../const/tender').measureType;

module.exports = app => {
    class StagePos extends app.BaseService {
        /**
         * 构造函数
         *
         * @param {Object} ctx - egg全局变量
         * @return {void}
         */
        constructor(ctx) {
            super(ctx);
            this.tableName = 'stage_pos';
            this.qtyFields = ['contract_qty', 'qc_qty']
        }

        /**
         * 查询期计量最后审核人数据
         * @param {Number} tid - 标段id
         * @param {Number} sid - 期id
         * @param {Number|Array} pid - 部位明细id（可以为空）
         * @returns {Promise<*>}
         */
        async getLastestStageData(tid, sid, pid) {
            let pidSql = pid ? (' And Pos.pid in (' + (pid instanceof Array ? pid.join(', ') : pid) + ')') : '';
            const sql = 'SELECT * FROM ' + this.tableName + ' As Pos ' +
                '  INNER JOIN ( ' +
                '    SELECT MAX(`times`) As `times`, MAX(`order`) As `order`, `pid` From ' + this.tableName +
                '      GROUP BY `pid`' +
                '  ) As MaxFilter ' +
                '  ON Pos.times = MaxFilter.times And Pos.order = MaxFilter.order And Pos.pid = MaxFilter.pid' +
                '  WHERE Pos.tid = ? And Pos.sid = ?' + pidSql;
            const sqlParam = [tid, sid];
            if (!pid) {
                return await this.db.query(sql, sqlParam);
            } else if (pid instanceof Array) {
                return await this.db.query(sql, sqlParam);
            } else {
                return await this.db.queryOne(sql, sqlParam);
            }
        }
        /**
         * 查询 某期 某轮审批 某审核人数据
         * @param {Number} tid - 标段id
         * @param {Number} sid - 期id
         * @param {Number} times - 期第几轮审批
         * @param {Number} order - 审核人顺序
         * @param {Number|Array|Null} pid - 部位明细id - 为空则查询全部
         * @returns {Promise<*>}
         */
        async getAuditorStageData(tid, sid, times, order, pid) {
            let pidSql;
            if (pid instanceof Array) {
                pidSql = pid.length > 0 ? ' And Pos.pid in (' + pid.join(', ') + ')' : '';
            } else {
                pidSql = pid ? 'And Pos.pid = ' + pid.toString() : '';
            }
            const sql = 'SELECT * FROM ' + this.tableName + ' As Pos ' +
                '  INNER JOIN ( ' +
                '    SELECT MAX(`times`) As `times`, MAX(`order`) As `order`, `pid` From ' + this.tableName +
                '      WHERE `times` <= ? AND `order` <= ?' +
                '      GROUP BY `pid`' +
                '  ) As MaxFilter ' +
                '  ON Pos.times = MaxFilter.times And Pos.order = MaxFilter.order And Pos.pid = MaxFilter.pid' +
                '  WHERE Pos.tid = ? And Pos.sid = ?' + pidSql;
            const sqlParam = [times, order, tid, sid];
            if (!pid) {
                return await this.db.query(sql, sqlParam);
            } else if (pid instanceof Array) {
                return await this.db.query(sql, sqlParam);
            } else {
                return await this.db.queryOne(sql, sqlParam);
            }
        }

        /**
         * 新增部位明细数据（仅供updateStageData调用）
         *
         * @param transaction - 事务
         * @param data - 新增数据
         * @returns {Promise<{}>}
         * @private
         */
        async _addStagePosData(transaction, data) {
            const bills = await this.ctx.service.ledger.getDataById(data.lid);
            const precision = this.ctx.helper.findPrecision(this.ctx.tender.info.precision, bills.unit);
            const  result = {};
            // 在主表pos中新增数据
            const p = JSON.parse(JSON.stringify(data.updateData));
            p.tid = this.ctx.tender.id;
            p.add_stage = this.ctx.stage.id;
            p.add_times = this.ctx.stage.times;
            p.add_user = this.ctx.session.sessionUser.accountId;
            if (p.contract_qty) { delete p.contract_qty; }
            if (p.qc_qty) { delete p.qc_qty; }
            if (p.postil) { delete p.postil; }
            if (p.quantity) {
                p.quantity = this.round(p.quantity, precision.value);
            }
            const addRst = await transaction.insert(this.ctx.service.pos.tableName, data.updateData);
            p.id = addRst.insertId;
            result.pos = p.id;
            // 如果存在复核数据，更新计算主表清单
            if (p.quantity) {
                await this.ctx.service.ledger.calc(this.ctx.tender.id, p.lid, transaction);
                result.ledger = p.lid;
            }
            // 如果存在本期计算数据，更新计算清单本期计量数据
            if (data.contract_qty || data.qc_qty || data.postil) {
                const ps = {
                    pid: p.id,
                    lid: p.lid,
                    tid: this.ctx.tender.id,
                    sid: this.ctx.stage.id,
                    said: this.ctx.session.sessionUser.accountId,
                    times: this.ctx.stage.times,
                    order: 0,
                };
                if (data.contract_qty) { ps.contract_qty = this.round(data.contract_qty, precision.value); }
                if (data.qc_qty) { ps.qc_qty = this.round(data.qc_qty, precision.value); }
                if (data.postil) { ps.postil = data.postil; }
                await transaction.insert(ps);
                await this.ctx.service.stageBills.calc(ctx.tender.id, ctx.stage.id, ps.lid, transaction);
                result.stageUpdate = true;
            }
            return result;
        }

        /**
         * 更新部位明细数据（仅供updateStageData调用）
         *
         * @param transaction - 事务
         * @param data - 更新数据(允许一次性提交多条)
         * @returns {Promise<{ledger: Array, pos: Array}>}
         * @private
         */
        async _updateStagePosData(transaction, data) {
            let bills, precision;
            const result = {ledger: [], pos: [], stageUpdate: true};
            const datas = data instanceof Array ? data : [data];
            const orgStagePos = await this.getLastestStageData(this.ctx.tender.id, this.ctx.stage.id, this._.map(datas, 'pid'));
            const userOrder = this.ctx.stage.curAuditor ? this.ctx.stage.curAuditor.order : 0;
            for (const d of datas) {
                if (!bills || bills.id !== data.lid) {
                    bills = await this.ctx.service.ledger.getDataById(data.lid);
                    precision = this.ctx.helper.findPrecision(this.ctx.tender.info.precision, bills.unit);
                }
                const osp = this._.find(orgStagePos, function (p) { return p.pid === d.pid; });
                this.ctx.helper.checkFieldPrecision(d, this.qtyFields, precision.value);
                if (osp && osp.times === this.ctx.stage.times && osp.order === userOrder) {
                    await transaction.update(this.tableName, d, {where: {id: osp.id}});
                } else {
                    console.log(osp);
                    d.tid = this.ctx.tender.id;
                    d.sid = this.ctx.stage.id;
                    d.said = this.ctx.session.sessionUser.accountId;
                    d.times = this.ctx.stage.times;
                    d.order = userOrder;
                    await transaction.insert(this.tableName, d);
                }
                result.pos.push(d.pid);
                if ((d.contract_qty === undefined || d.qc_qty === undefined) && (result.ledger.indexOf(d.lid) === -1)) {
                    result.ledger.push(d.lid);
                }
            }
            for (const lid of result.ledger) {
                await this.ctx.service.stageBills.calc(this.ctx.tender.id, this.ctx.stage.id, lid, transaction);
            }
            return result;
        }

        /**
         * 删除部位明细数据（仅供updateStageData调用）
         *
         * @param transaction - 事务
         * @param data - 删除的部位明细(允许一次提醒多条，也允许跨清单（但前端操作不允许）)
         * @returns {Promise<{}>}
         * @private
         */
        async _deleteStagePosData(transaction, data) {
            const result = {};
            const pos = await this.ctx.service.pos.getPosData({tid: this.ctx.tender.id, id: data});
            if (pos instanceof Array) {
                for (const p of pos) {
                    if (p.add_stage !== this.ctx.stage.id || p.add_times !== this.ctx.stage.times || p.add_user !== this.ctx.session.sessionUser.accountId) {
                        throw '您无权删除该数据';
                    }
                }
            } else if (pos.add_stage !== this.ctx.stage.id || pos.add_times !== this.ctx.stage.times || pos.add_user !== this.ctx.session.sessionUser.accountId) {
                throw '您无权删除该数据';
            }
            const ledgerIds = this._.map(pos, 'lid');
            // 删除部位明细
            await transaction.delete(this.ctx.service.pos.tableName, {tid: this.ctx.tender.id, id: data});
            for (const lid of ledgerIds) {
                await this.ctx.service.ledger.calc(tid, lid, transaction);
            }
            // 删除部位明细计量数据
            await transaction.delete(this.tableName, {tid: this.ctx.tender.id, lid: data});
            for (const lid of ledgerIds) {
                await this.ctx.service.stageBills.calc(this.ctx.tender.id, this.ctx.stage.id, lid, transaction);
            }
            // 获取需要更新的数据
            result.ledger = ledgerIds;
            result.stageUpdate = true;
            return result;
        }

        /**
         * 根据前端提交数据，更新并计算
         *
         * @param data
         * @returns {Promise<{ledger: {}, pos: {}}>}
         */
        async updateStageData(data) {
            let refreshData;
            const transaction = await this.db.beginTransaction();
            try {
                if ((data.updateType === 'add' || data.upateType === 'delete') && this.ctx.tender.measure_type === measureType.tz) {
                    throw '台账模式下，不可在计量中新增或删除部位明细，如需操作，请进行台账修订';
                }
                if (data.updateType === 'add') {
                    refreshData = await this._addStagePosData(transaction, data.updateData);
                } else if (data.updateType === 'update') {
                    refreshData = await this._updateStagePosData(transaction, data.updateData);
                } else if (data.updateType === 'delete') {
                    if (!data.updateData || data.updateData.length === 0) {
                        throw '提交数据错误';
                    }
                    refreshData = await this._deleteStagePosData(transaction, data.updateData);
                } else {
                    throw '提交数据错误';
                }
                await transaction.commit();
            } catch (err) {
                await transaction.rollback();
                throw err;
            }

            try {
                const result = {ledger: {}, pos: {}};
                if (refreshData.ledger && refreshData.ledger.length > 0) {
                    result.ledger.bills = await this.ctx.service.ledger.getDataByIds(refreshData.ledger);
                    if (refreshData.stageUpdate) {
                        result.ledger.curStageData = await await this.ctx.service.stageBills.getLastestStageData(this.ctx.tender.id, this.ctx.stage.id, refreshData.ledger);
                    }
                }
                if (refreshData.pos && refreshData.pos.length > 0) {
                    result.pos.pos = await this.ctx.service.pos.getPosData({id: refreshData.pos});
                    if (refreshData.stageUpdate) {
                        result.pos.curStageData = await this.getLastestStageData(this.ctx.tender.id, this.ctx.stage.id, refreshData.pos);
                    }
                }
                return result;
            } catch(err) {
                throw '获取数据异常，请刷新页面。';
            }
        }

        /**
         * 统计清单下部位明细合计
         * @param {Number} tid - 标段id
         * @param {Number} sid - 期id
         * @param {Number} lid - 清单节点id
         * @param transaction - 事务（不为空则在事务中查询，反之在数据库中查询）
         * @returns {Promise<*>}
         */
        async getPosGather(tid, sid, lid, transaction) {
            const calcQtySql = 'SELECT SUM(`contract_qty`) As `contract_qty`, SUM(`qc_qty`) As `qc_qty` FROM (' +
                '  SELECT `contract_qty`, `qc_qty` FROM ' + this.ctx.service.stagePos.tableName + ' As Pos ' +
                '    INNER JOIN (' +
                '      SELECT MAX(`times`) As `times`, MAX(`order`) As `order`, `pid` ' +
                '        FROM ' + this.ctx.service.stagePos.tableName +
                '        WHERE `tid` = ? And sid = ? And `lid` = ? ' +
                '        GROUP BY `pid`' +
                '    ) As MaxFilter ' +
                '    ON Pos.times = MaxFilter.times And Pos.order = MaxFilter.order And Pos.pid = MaxFilter.pid ' +
                '    WHERE Pos.tid = ? And Pos.sid = ? And Pos.lid = ?' +
                '  ) As Gather';
            const param = [tid, sid, lid];
            const sqlParam = param.concat(param);
            if (transaction) {
                return await transaction.queryOne(calcQtySql, sqlParam);
            } else {
                return await this.db.queryOne(calcQtySql, sqlParam);
            }
        }
    }

    return StagePos;
};